#!/bin/python3
import tkinter as tk
from tkinter import scrolledtext, messagebox, ttk
import subprocess
import threading
from PIL import Image, ImageTk
import os

class UpgradeApp:
    def __init__(self, root):
        self.root = root
        self.root.title("系统升级")
        self.steps = [
            "升级系统",
            "安装新桌面",
            "卸载Mate桌面（可选）",
            "重启"
        ]
        self.current_step = 0

        # 设置窗口大小
        self.root.minsize(800, 400)
        self.expanded = False

        self.center_window(800, 400)

        # 设置窗口半透明
        self.root.attributes('-alpha', 0.95)  # 95% opaque, 5% transparent

        # 使用ttk的主题
        self.style = ttk.Style()
        self.style.theme_use('clam')

        # 自定义样式
        self.style.configure('TButton', font=('Helvetica', 12), padding=6, background='#0052cc', foreground='#ffffff')
        self.style.configure('TLabel', font=('Helvetica', 10), background='white')
        self.style.configure('TFrame', background='white')

        # 创建主框架
        main_frame = ttk.Frame(self.root)
        main_frame.grid(row=0, column=0, sticky='nsew')
        self.root.grid_rowconfigure(0, weight=1)
        self.root.grid_columnconfigure(0, weight=1)

        # 左侧步骤列表框架
        steps_frame = ttk.Frame(main_frame)
        steps_frame.grid(row=0, column=0, sticky='nsew', padx=(0, 10))
        main_frame.grid_rowconfigure(0, weight=1)
        main_frame.grid_columnconfigure(0, weight=1)

        steps_label = ttk.Label(steps_frame, text="升级步骤", font=('Helvetica', 14, 'bold'))
        steps_label.pack(anchor=tk.W, pady=(0, 10))

        self.steps_listbox = tk.Listbox(steps_frame, font=('Helvetica', 12), height=len(self.steps))
        self.steps_listbox.pack(fill=tk.BOTH, expand=True)
        for step in self.steps:
            self.steps_listbox.insert(tk.END, step)

        # 右侧执行框架
        action_frame = ttk.Frame(main_frame)
        action_frame.grid(row=0, column=1, sticky='nsew')
        main_frame.grid_rowconfigure(0, weight=1)
        main_frame.grid_columnconfigure(1, weight=2)

        # 文件复制进度条
        self.progress = ttk.Progressbar(action_frame, orient=tk.HORIZONTAL, mode='indeterminate')
        self.progress.pack(fill=tk.X, pady=(10, 5))

        # 创建一个 Text 小部件用于显示命令输出
        self.output_text = scrolledtext.ScrolledText(action_frame, wrap=tk.WORD, font=('Helvetica', 10), height=10)
        self.output_text.pack(fill='both', expand=True, pady=(10, 10))

        # 确认升级/下一步按钮框架
        button_frame = ttk.Frame(self.root)
        button_frame.grid(row=1, column=0, columnspan=2, sticky='ew', pady=(10, 10))

        self.next_button = ttk.Button(button_frame, text="开始升级", command=self.next_step)
        self.next_button.pack(side=tk.RIGHT, padx=(5, 0))

        self.root.grid_rowconfigure(1, weight=0)  # 确保按钮框架不会占据多余的空间

        command = "dpkg -s task-loongnix-desktop"
        process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        process.wait()
        if process.returncode == 0:
            self.next_button.config(state=tk.DISABLED)
            self.output_text.insert(tk.END, "您已经安装了loongnix通用版桌面，无需进行升级操作.\n")

        command = "dpkg -s artoon-desktop"
        process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        process.wait()
        if process.returncode == 0:
            self.next_button.config(state=tk.DISABLED)
            self.output_text.insert(tk.END, "您已经安装了cartoon定制版系统，该系统不支持升级到通用版.\n")


    def center_window(self, width, height):
        # 获取屏幕宽度和高度
        screen_width = self.root.winfo_screenwidth()
        screen_height = self.root.winfo_screenheight()

        # 计算窗口的 x 和 y 坐标
        x = (screen_width // 2) - (width // 2)
        y = (screen_height // 2) - (height // 2)

        self.root.geometry(f'{width}x{height}+{x}+{y}')

    def next_step(self):
        if self.current_step == 0:
            if messagebox.askyesno("确认", "升级到新桌面可能会导致当前桌面显示异常，应用不能使用等问题，建议您备份好数据后进行升级，你确定要升级系统吗？"):
                self.output_text.insert(tk.END, "正在升级系统...\n")
                self.run_step("DEBIAN_FRONTEND=noninteractive apt update && DEBIAN_FRONTEND=noninteractive apt upgrade -y")
        elif self.current_step == 1:
            self.run_step("DEBIAN_FRONTEND=noninteractive apt install -y task-loongnix-desktop loonggpu-driver && ln -sf /lib/systemd/system/sddm.service /etc/systemd/system/display-manager.service && update-alternatives --install /usr/bin/x-session-manager x-session-manager /usr/bin/startplasma-x11 60")
        elif self.current_step == 2:
            if messagebox.askyesno("Response", "安装桌面成功,是否卸载MATE桌面环境"):
                self.run_step("DEBIAN_FRONTEND=noninteractive apt remove --purge -y lightdm mate-* && DEBIAN_FRONTEND=noninteractive apt install -y kde-settings-daemon")
            else:
                self.run_step("echo 不卸载MATE桌面环境，下一步")
 #               self.run_step("DEBIAN_FRONTEND=noninteractive apt install -y kde-settings-daemon")
        elif self.current_step == 3:
            messagebox.showinfo("完成", "即将重启桌面")
            self.run_step("/sbin/reboot")
        else:
            messagebox.showinfo("完成", "所有步骤已完成")

    def run_step(self, command):
        self.next_button.config(state=tk.DISABLED)
        self.output_text.insert(tk.END, f"执行命令: {command}\n")
        threading.Thread(target=self.run_command_and_output, args=(command,)).start()

    def run_command_and_output(self, command):
        try:

            self.progress.start()
            process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

            for line in process.stdout:
                self.output_text.insert(tk.END, line)
                self.output_text.see(tk.END)

            for line in process.stderr:
                self.output_text.insert(tk.END, line)
                self.output_text.see(tk.END)

            process.wait()
            if process.returncode == 0:
                self.output_text.insert(tk.END, f"\n命令执行成功: {command}\n")
                self.steps_listbox.itemconfig(self.current_step, {'bg': 'green', 'fg': 'white'})
            else:
                self.output_text.insert(tk.END, f"\n命令执行失败: {command}\n")
                self.steps_listbox.itemconfig(self.current_step, {'bg': 'red', 'fg': 'white'})
                self.root.update_idletasks()  # 强制刷新以立即显示红色背景
                raise Exception("请排查升级执行记录，如需要帮助请联系支持邮箱service@loongnix.cn")

            self.current_step += 1

            if self.current_step < len(self.steps):
                self.steps_listbox.selection_clear(0, tk.END)

                self.steps_listbox.itemconfig(self.current_step, {'bg': '#0052cc', 'fg': 'white'})  # 高亮当前步骤

#                for i in range(len(self.steps)):
#                    if i == self.current_step:
#                        self.steps_listbox.itemconfig(i, {'bg': '#0052cc', 'fg': 'white'})  # 高亮当前步骤
#                    else:
#                        self.steps_listbox.itemconfig(i, {'bg': 'white', 'fg': 'black'})  # 取消其他步骤的高亮

                self.next_button.config(text="下一步")
                self.next_button.config(state=tk.NORMAL)
            else:
                self.next_button.config(state=tk.DISABLED)

            self.progress.stop()
            self.output_text.see(tk.END)

        except Exception as e:
#            self.output_text.insert(tk.END, f"\n执行命令时出错: {e}\n")
            self.output_text.see(tk.END)
            self.steps_listbox.itemconfig(self.current_step, {'bg': 'white', 'fg': 'black'})  # 取消其他步骤的高亮
            self.steps_listbox.itemconfig(self.current_step, {'bg': 'red', 'fg': 'white'})  # 确保背景颜色设置为红色
            self.root.update_idletasks()  # 强制刷新以立即显示红色背景
            self.next_button.config(state=tk.DISABLED)
            self.progress.stop()


if __name__ == "__main__":
    root = tk.Tk()
    app = UpgradeApp(root)
    root.mainloop()

